"use strict";Object.defineProperty(exports, "__esModule", {value: true});// src/index.ts
var _unplugin = require('unplugin');
var _pluginutils = require('@rollup/pluginutils');





var _common = require('@vue-macros/common');
var _api = require('@vue-macros/api');

// src/core/index.ts








// src/core/define-emit.ts


// src/core/constants.ts

var PROPS_VARIABLE_NAME = `${_common.HELPER_PREFIX}props`;
var EMIT_VARIABLE_NAME = `${_common.HELPER_PREFIX}emit`;

// src/core/define-emit.ts
function mountEmits(emits) {
  const isAllWithoutOptions = emits.every(({ validator }) => !validator);
  if (isAllWithoutOptions) {
    return `[${emits.map(({ name: name2 }) => JSON.stringify(name2)).join(", ")}]`;
  }
  return `{
      ${emits.map(
    ({ name: name2, validator }) => `${JSON.stringify(name2)}: ${validator || `null`}`
  ).join(", ")}
    }`;
}
function transformDefineEmit({ setupAst, offset, s }) {
  const emits = [];
  _common.walkAST.call(void 0, setupAst, {
    enter(node) {
      if (_common.isCallOf.call(void 0, node, _common.DEFINE_EMIT)) {
        const [name2, validator] = node.arguments;
        if (name2.type !== "StringLiteral") {
          throw new Error(
            `The first argument of ${_common.DEFINE_EMIT} must be a string literal.`
          );
        }
        emits.push({
          name: name2.value,
          validator: validator ? s.sliceNode(validator, { offset }) : void 0
        });
        s.overwriteNode(
          node,
          `(...args) => ${EMIT_VARIABLE_NAME}(${JSON.stringify(
            name2.value
          )}, ...args)`,
          { offset }
        );
      }
    }
  });
  if (emits.length > 0) {
    s.prependLeft(
      offset,
      `
const ${EMIT_VARIABLE_NAME} = defineEmits(${mountEmits(emits)})
`
    );
  }
}

// src/core/define-prop.ts












async function mountProps({ id, scriptSetup, setupAst, isProduction }, props) {
  const isAllWithoutOptions = props.every(
    ({ definition, typeParameter }) => !definition && !typeParameter
  );
  if (isAllWithoutOptions) {
    return `[${props.map(({ name: name2 }) => JSON.stringify(name2)).join(", ")}]`;
  }
  let propsString = "{\n";
  for (const { name: name2, definition, typeParameter } of props) {
    propsString += `  ${JSON.stringify(name2)}: `;
    if (definition) {
      propsString += definition;
    } else if (typeParameter && !isProduction) {
      const resolved = await _api.resolveTSReferencedType.call(void 0, {
        scope: {
          filePath: id,
          content: scriptSetup.content,
          ast: setupAst.body
        },
        type: typeParameter
      });
      propsString += resolved ? `{ type: ${_api.toRuntimeTypeString.call(void 0, await _api.inferRuntimeType.call(void 0, resolved))} }` : "null";
    } else {
      propsString += "null";
    }
    propsString += ",\n";
  }
  propsString += "}";
  return propsString;
}
async function transformDefineProp(options) {
  const { setupAst, offset, s } = options;
  const props = [];
  _common.walkAST.call(void 0, setupAst, {
    enter(node) {
      var _a;
      if (_common.isCallOf.call(void 0, node, _common.DEFINE_PROP)) {
        const [name2, definition] = node.arguments;
        if (name2.type !== "StringLiteral") {
          throw new Error(
            `The first argument of ${_common.DEFINE_PROP} must be a string literal.`
          );
        }
        props.push({
          name: name2.value,
          definition: definition ? s.sliceNode(definition, { offset }) : void 0,
          typeParameter: (_a = node.typeParameters) == null ? void 0 : _a.params[0]
        });
        s.overwriteNode(
          node,
          `${_common.HELPER_PREFIX}computed(() => ${PROPS_VARIABLE_NAME}[${JSON.stringify(
            name2.value
          )}])`,
          { offset }
        );
      }
    }
  });
  if (props.length > 0) {
    _common.importHelperFn.call(void 0, s, offset, "computed", "vue");
    s.prependLeft(
      offset,
      `
const ${PROPS_VARIABLE_NAME} = defineProps(${await mountProps(
        options,
        props
      )})
`
    );
  }
}

// src/core/index.ts
async function transformDefineSingle(code, id, isProduction) {
  const { scriptSetup, getSetupAst } = _common.parseSFC.call(void 0, code, id);
  if (!scriptSetup)
    return;
  const offset = scriptSetup.loc.start.offset;
  const s = new (0, _common.MagicString)(code);
  const setupAst = getSetupAst();
  const options = {
    id,
    s,
    offset,
    scriptSetup,
    setupAst,
    isProduction
  };
  if (code.includes(_common.DEFINE_PROP)) {
    await transformDefineProp(options);
  }
  if (code.includes(_common.DEFINE_EMIT)) {
    await transformDefineEmit(options);
  }
  return _common.getTransformResult.call(void 0, s, id);
}

// src/index.ts
function resolveOption(options, framework) {
  const version = options.version || _common.detectVueVersion.call(void 0, );
  return {
    include: [_common.REGEX_VUE_SFC, _common.REGEX_SETUP_SFC].concat(
      version === 2 && framework === "webpack" ? _common.REGEX_VUE_SUB : []
    ),
    isProduction: process.env.NODE_ENV === "production",
    ...options,
    version
  };
}
var name = "unplugin-vue-single-define";
var src_default = _unplugin.createUnplugin.call(void 0, 
  (userOptions = {}, { framework }) => {
    const options = resolveOption(userOptions, framework);
    const filter = _pluginutils.createFilter.call(void 0, options.include, options.exclude);
    const { resolve, handleHotUpdate } = _api.RollupResolve.call(void 0, );
    return {
      name,
      enforce: "pre",
      buildStart() {
        if (framework === "rollup" || framework === "vite") {
          _api.setResolveTSFileIdImpl.call(void 0, resolve(this));
        }
      },
      transformInclude(id) {
        return filter(id);
      },
      transform(code, id) {
        try {
          return transformDefineSingle(code, id, options.isProduction);
        } catch (err) {
          this.error(`${name} ${err}`);
        }
      },
      vite: {
        configResolved(config) {
          options.isProduction = config.isProduction;
        },
        handleHotUpdate
      }
    };
  }
);



exports.src_default = src_default;
